//
// Copyright (c) 2005 Yoan Blanc, HE-ARC Ingnierie (Le Locle)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
// Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
// OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

package ch.hearc.I04_05s08.GUI.Graph;

import java.awt.*;
import java.awt.geom.*;

/**
 *	Arrow between nodes
 *
 *	@author Blanc, Yoan
 *	@hearc.copyright 2005 Yoan Blanc, HE-ARC Ingnierie
 *	@hearc.licence <a href="http://opensource.org/licenses/mit-license.php">MIT</a>
 */

public abstract class Arrow	extends Element
{
	/**
	 *	The line
	 */
	private Line2D line;
	
	/**
	 *	The rectangle (background of the title)
	 */
	private Rectangle2D rectangle;
	
	/**
	 *	Nodes connected
	 */
	private Node[] nodes;
	
	/**
	 *	Point of contact with the nodes
	 */
	private Point2D[] contactPoints;
	
	/**
	 *	The arrow
	 */
	private Polygon arrow;
	
	//--
	
	/**
	 *	Build a new arrow
	 *
	 *	@param title		the title
	 *	@param from		the node where it comes
	 *	@param to		the node where it goes
	 */
	public Arrow(String title, Node from, Node to)
	{
		super(title);
		
		nodes = new Node[]{from, to};	
		contactPoints = new Point2D[2];
	}
	
	/**
	 *	Build a new arrow without a title
	 *
	 *	@param from		the node where it comes
	 *	@param to		the node where it goes
	 */
	public Arrow(Node from, Node to)
	{
		this(null, from, to);
	}
	
	/**
	 *	Build a new arrow without anything
	 */
	public Arrow()
	{
		this(null, null, null);
	}
	
	/**
	 *	Build a new arrow from another
	 *
	 *	@param a		arrow
	 */
	public Arrow(Arrow a)
	{
		super(a);
		
		nodes = a.getNodes();
		contactPoints = new Point2D[2];
	}
	
	/**
	 *	Set the nodes
	 *
	 *	@param from		the node where it comes
	 *	@param to		the node where it goes
	 *	@return true
	 */
	public boolean setNodes(Node from, Node to)
	{
		if(from != null && to != null)
		{
			nodes = new Node[]{from, to};	
			return true;
		}
		else
		{
			return false;
		}
	}
	
	/**
	 *	Get the nodes
	 *
	 *	@return			the nodes
	 */
	public Node[] getNodes()
	{
		return nodes;
	}
	
	/**
	 *	If the line is near (3) the point
	 *
	 *	@param p		the point
	 *	@return		if the point is close to the Arrow
	 */
	public boolean contains(Point2D p)
	{
		boolean contains = false;
		
		if(line != null)
			contains |= line.ptSegDist(p)<3;
		
		if(rectangle != null)
			contains |= rectangle.contains(p);
		
		if(arrow != null)
			contains |= arrow.contains(p);
		
		return contains;
	}
	
	/**
	 *	Get the point of contact with other nodes
	 *
	 *	@return points where the line touches the nodes
	 */
	public Point2D[] getContactPoints()
	{
		return contactPoints;
	}
	
	/**
	 *	Get the line
	 *
	 *	@return the line
	 */
	public Line2D getLine()
	{
		return line;
	}
	
	/**
	 *	Caluculate the line
	 *
	 *	@param g2d	Graphic context
	 */
	public void calcLine(Graphics2D g2d)
	{		
		if(nodes[0] != null && nodes[1] != null)
		{
			contactPoints = new Point2D[]{
				nodes[0].getProximityPoint((Point2D)nodes[1].getGravityPoint(g2d), g2d),
				nodes[1].getProximityPoint((Point2D)nodes[0].getGravityPoint(g2d), g2d)
			};
			
			line = new Line2D.Double(contactPoints[0], contactPoints[1]);
			
			if(getTitle() != null && getTitle().length() > 0)
			{
				double width = g2d.getFontMetrics().stringWidth(getTitle());
				double height = g2d.getFontMetrics().getHeight();
				
				rectangle = new Rectangle2D.Double(
					line.getBounds().getCenterX() - width/2. - getPadding().getWidth(),
					line.getBounds().getCenterY() - height/2. - getPadding().getHeight(),
					width + 2.*getPadding().getWidth(),
					height + 2.*getPadding().getHeight()
				);
			}
		}
	}
	
	/**
	 *	Draw
	 *
	 *	@param g2d	Graphic context
	 */
	public void draw(Graphics2D g2d)
	{
		calcLine(g2d);
		
		if(line != null)
		{
			drawLine(g2d);
			
			if(getTitle() != null && getTitle().length() > 0)
				drawTitle(g2d);
		}
	}
	
	/**
	 *	Draw the title's Node
	 *
	 *	@param g2d	Graphic context
	 */
	public void drawTitle(Graphics2D g2d)
	{
		calcLine(g2d);
		
		g2d.setColor(getBackgroundColor());
		g2d.fill(rectangle);
		
		g2d.setColor(getForegroundColor());
		g2d.drawString(
			getTitle(),
			(float)(rectangle.getX() + getPadding().getWidth()),
			(float)(rectangle.getCenterY() + getPadding().getHeight())
		);
	}
	
	/**
	 *	Drawing the line between the two nodes
	 *
	 *	@param g2d	Graphic context
	 */
	public void drawLine(Graphics2D g2d)
	{
		g2d.setColor(getForegroundColor());
		g2d.draw(line);
	}
	
	/**
	 *	Get the angle of the line (between 0 and 2*PI)
	 *
	 *	@return the angle
	 */
	public double getAngle()
	{
		double angle = 0;
		
		if(line != null)
		{
			Point2D to = getLine().getP2();
			Point2D from = getLine().getP1();
			
			if((to.getX() - from.getX()) != 0)
			{
				double p = (to.getY() - from.getY())/(to.getX() - from.getX());
				angle = Math.atan(p);
				
				if(to.getX() > from.getX())
				{
					angle = Math.PI + angle;
				}
			}
			else
			{
				angle = (to.getY() > from.getY()) ? -Math.PI/2. : Math.PI/2.;
			}
		}
		
		return angle;
	}
	
	/**
	 *	Define the arrow
	 *
	 *	@param arrow		the arrow
	 */
	public void setArrow(Polygon arrow)
	{
		this.arrow = arrow;
	}
	
	/**
	 *	Disconnect the arrow from its nodes
	 *
	 *	@return the success of operation
	 */
	public boolean unconnect()
	{
		nodes = new Node[2];
		return true;
	}
}
