//
// Copyright (c) 2005 Yoan Blanc, HE-ARC Ingnierie (Le Locle)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
// Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
// OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

package ch.hearc.I04_05s08.GUI.Graph;

import javax.swing.*;
import java.util.LinkedList;
import java.util.Iterator;
import java.awt.*;
import java.awt.geom.*;
import ch.hearc.I04_05s08.Ontology.OntFile;

/**
 *	Graph
 *
 *	@author Blanc, Yoan
 *	@hearc.copyright 2005 Yoan Blanc, HE-ARC Ingnierie
 *	@hearc.licence <a href="http://opensource.org/licenses/mit-license.php">MIT</a>
 */

public abstract class GraphPanel extends JPanel
{
	/**
	 *	List of nodes
	 */
	private LinkedList nodes;
	
	/**
	 *	List of arrows
	 */
	private LinkedList arrows;
	
	/**
	 *	List of lists
	 */
	private LinkedList[] elements;
	
	/**
	 *	Current selected Element
	 */
	private Element selectedElement;
		
	/**
	 *	Last selected Element
	 */
	private Element oldElement;
	
	/**
	 *	Offset of the point inside the Node
	 */
	private Point2D offset;
	
	/**
	 *	Temporary line before arrow
	 */
	private Line2D line;
	
	/**
	 *	Mode of drawing
	 */
	private int mode;
	
	/**
	 *	The element that will be drawing
	 */
	private Element drawingElement;
	
	/**
	 *	Ontology file
	 */
	private OntFile file;
	
	/**
	 *	Mode drawing an arrow
	 */
	public static final int NO_MODE = 0;
	
	/**
	 *	Mode drawing an arrow
	 */
	public static final int ARROW_MODE = 1;
		
	/**
	 *	Mode drawing a node
	 */
	public static final int NODE_MODE = 2;
	
	/**
	 *	Build the graph
	 */
	public GraphPanel()
	{
		setLayout(null);
		setBackground(Color.white);
		
		nodes = new LinkedList();
		arrows = new LinkedList();
		
		elements = new LinkedList[]{nodes, arrows};
		
		mode = 0;
	}
	
	/**
	 *	Add a node
	 *
	 *	@param n		Node
	 */
	public boolean add(Node n)
	{
		if(n!=null && !nodes.contains(n))
		{
			return nodes.add(n);
		}
		else
		{
			return false;
		}
	}
	
	/**
	 *	Add nodes
	 *
	 *	@param n		Nodes
	 */
	public boolean add(Node[] n)
	{
		boolean val = true;
		
		for(int i=0; i<n.length; i++)
			if(n[i]!=null && !nodes.contains(n[i]))
			{
				val &= add(n[i]);
			}
		
		return val;
	}
	
	/**
	 *	Add an arrow
	 *
	 *	@param a		Arrow
	 */
	public boolean add(Arrow a)
	{
		if(a != null && !arrows.contains(a))
		{
			return arrows.add(a);
		}
		else
		{
			return false;
		}
	}
	
	/**
	 *	Add arrows
	 *
	 *	@param a		Arrow
	 */
	public boolean add(Arrow[] a)
	{
		boolean val = true;
		
		for(int i=0; i<a.length; i++)
			val &= add(a[i]);
		
		return val;
	}
	
	/**
	 *	Add an Element to the graph
	 *
	 *	@param e		the element
	 */
	public boolean add(Element e)
	{
		boolean val;
		
		if(e instanceof Node)
		{
			((Arrow)e).setNodes((Node)oldElement, (Node)selectedElement);
			val = add((Node)e);
		}
		else if(e instanceof Arrow)
		{
			val = add((Arrow)e);
		}
		else
		{
			val = false;
		}
		
		invalidate();
		repaint();
		
		return val;
	}
	
	/**
	 *	Delete an arrow
	 *
	 *	@param a arrow
	 */
	public boolean del(Arrow a)
	{
		return arrows.remove(a);
	}
	
	/**
	 *	Delete some arrows
	 *
	 *	@param a arrows
	 */
	public boolean del(Arrow[] a)
	{
		boolean val=true;
		
		for(int i=0;i<a.length;i++)
			val &= del(a[i]);
		
		return val;
	}
	
	/**
	 *	Delete a node (and connected arrows)
	 *
	 *	@param n	node to delete
	 */
	public boolean del(Node n)
	{
		boolean val=true;
		
		// arrows to delete
		Arrow[] a = new Arrow[arrows.size()];
		int i=0;
		
		if(a.length > 0)
		{
			for(Iterator iter = arrows.iterator(); iter.hasNext();)
			{
				Arrow arr = (Arrow)iter.next();
				Node[] ns = arr.getNodes();
				
				if(ns[0] == n || ns[1] == n)
				{
					a[i] = arr;
					i++;
				}
			}
			
			for(int k=0;k<i;k++)
			{
				val &= del((Element)a[k]);
			}
		}
		
		return nodes.remove(n) && val;
	}
	
	private void showme()
	{
		for(int i = 0; i<elements.length; i++)
		{
			int k=0;
			for(Iterator j = elements[i].iterator(); j.hasNext();)
			{
				System.out.println(i +","+k+ " : "+(Element)j.next());
				k++;
			}
		}
	}
	
	/**
	 *	Delete some nodes (and connected arrows)
	 *
	 *	@param n	node to delete
	 */
	public boolean del(Node[] n)
	{
		boolean val = true;
		
		for(int i=0; i<n.length; i++)
			val &= del(n[i]);
		
		return val;
	}
	
	/**
	 *	Delete a element (node or arrow)
	 *
	 *	@param e		element
	 */
	public boolean del(Element e)
	{
		boolean val;
		
		if(e instanceof Node)
		{
			val = del((Node)e);
		}
		else if(e instanceof Arrow)
		{
			val = del((Arrow)e);
		}
		else
		{
			val = false;
		}
		
		redraw();
		
		return val;
	}
	
	/**
	 *	Get the file
	 *
	 *	@return a file
	 */
	public OntFile getFile()
	{
		return file;
	}
	
	/**
	 *	Set the file
	 *
	 *	@param file		a file
	 */
	public void setFile(OntFile file)
	{
		this.file = file;
	}
	
	/**
	 *	Paint the panel
	 *
	 *	@param g		Graphic context
	 */
	public void paintComponent(Graphics g)
	{
		super.paintComponent(g);
		
		Graphics2D g2 = (Graphics2D) g;
		
		// dessin dans le sens inverse des priorits (le plus prioritaire au-dessus)
		for(int i = elements.length-1; i>=0; i--)
		{
			for(Iterator j = elements[i].iterator(); j.hasNext();)
			{
				((Element)j.next()).draw(g2);
			}
		}
		
		if(mode == ARROW_MODE && line != null)
		{
			g2.draw(line);
		}
	}
	
	/**
	 *	Select the Element at the choosen point
	 *
	 *	@param p		the point clicked on screen
	 */
	public void select(Point2D p)
	{
		Object e;
		
		oldElement = selectedElement;
		selectedElement = null;
		offset = null;
		
		for(int i = 0; i<elements.length; i++)
		{
			for(Iterator j = elements[i].iterator(); j.hasNext();)
			{
				e = j.next();
				
				if(selectedElement == null && e != null && ((Element)e).contains(p))
				{
					selectedElement = (Element)e;
				}
				else
				{
					((Element)e).select(false);
				}
			}
			
			if(selectedElement != null)
			{
				// increase the z-index of the selected Element
				
				if(elements[i].contains(selectedElement))
				{
					elements[i].remove(selectedElement);
					elements[i].add(selectedElement);
				}
				
				selectedElement.select(true);
				
				if(selectedElement instanceof Node)
				{
					Point2D jp = ((Node)selectedElement).getPoint();
					
					offset = new Point2D.Double(
						p.getX() - jp.getX(),
						p.getY() - jp.getY()
					);
				}
			}
		}
		
		modeAction(p);
	}
	
	/**
	 *	Display a dialog box for the selected element
	 *
	 *	@param p		the point clicked on screen
	 */
	public void showDialog(Point2D p)
	{
		select(p);
		if(selectedElement != null)
			selectedElement.showDialog(this);
	}
	
	/**
	 *	Set the element that would be drawn
	 *
	 *	@param e			define the element that will be drawn
	 */
	public void setDrawingElement(Element e)
	{
		drawingElement = e;
	}
	
	/**
	 *	Do the actions for the modes (ARROW_MODE here)
	 *
	 *	@param p		the point clicked on screen
	 */
	public void modeAction(Point2D p)
	{
		switch (mode)
		{
			case ARROW_MODE:
				if(selectedElement != null && selectedElement instanceof Node && drawingElement instanceof Arrow)
				{
					if(oldElement == null)
					{
						line = new Line2D.Double(p,p);
					}
					else if(line != null && oldElement instanceof Node && oldElement != selectedElement)
					{
						add((Element)drawingElement);
						setMode(NO_MODE);
						
						line = null;
						selectedElement.select(false);
						selectedElement = null;
						
					}
					else
					{
						line = null;
						
						selectedElement.select(false);
						selectedElement = null;
					}
				}
				else
				{
					line = null;
				}
				break;
			case NODE_MODE:
				if(selectedElement == null && drawingElement instanceof Node)
				{
					((Node)drawingElement).setPos(p);
					add((Element)drawingElement);
					setMode(NO_MODE);
				}
				break;
		}
		
		redraw();
	}
	
	/**
	 *	Move the selected element to the point
	 *
	 *	@param p	the point
	 */
	public void moveTo(Point2D p)
	{
		// disabling the possibility of going below 0
		p.setLocation(Math.max(p.getX(),0), Math.max(p.getY(),0));
		
		if(line != null)
			line = new Line2D.Double(p,p);
		
		if(selectedElement != null  && selectedElement instanceof Node)
		{
			p.setLocation(
				Math.max(p.getX() - offset.getX(), 0),
				Math.max(p.getY() - offset.getY(), 0)
			);
			
			((Node)selectedElement).setPos(p);
		}
		
		redraw();
	}
	
	/**
	 *	Set the mouse position
	 */
	public void setMousePos(Point2D p)
	{
		if(mode == ARROW_MODE && line != null)
		{
			line.setLine(line.getP1(), p);
			
			redraw();
		}
	}
	
	/**
	 *	Get the size of the graph (used by the scroll area)
	 *
	 *	@return dimension of the graph
	 */
	public Dimension getPreferredSize() 
	{
		Rectangle2D bounds = new Rectangle2D.Double();
		
		for(Iterator i = nodes.iterator(); i.hasNext();)
		{
			Rectangle2D.union(bounds, ((Node)i.next()).getBounds(), bounds);
		}
		
		return new Dimension((int)bounds.getWidth(), (int)bounds.getHeight());
	}
	
	/**
	 *	Get the selected element
	 *
	 *	@return the element selected
	 */
	public Element getSelectedElement()
	{
		return selectedElement;
	}
	
	/**
	 *	Get the last selected element
	 *
	 *	@return the last element selected
	 */
	public Element getOldSelectedElement()
	{
		return oldElement;
	}
	
	/**
	 *	Set the drawing mode
	 *
	 *	@param mode		the mode
	 */
	public void setMode(int mode)
	{
		this.mode = mode;
		
		if(mode == NO_MODE)
		{
			drawingElement =null;
		}
	}
	
	/**
	 *	Get the drawing mode
	 *
	 *	@return the mode
	 */
	public int getMode()
	{
		return mode;
	}
	
	/**
	 *	Update the graph
	 */
	public void redraw()
	{
		invalidate();
		repaint();
		revalidate();
	}
	
	/**
	 *	Get the nodes
	 */
	public LinkedList getNodes()
	{
		return nodes;
	}
}
