//
// Copyright (c) 2005 Yoan Blanc, HE-ARC Ingnierie (Le Locle)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
// Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
// OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

package ch.hearc.I04_05s08.GUI.Graph;

import java.awt.Graphics2D;
import java.awt.Dimension;
import java.awt.Rectangle;
import java.awt.geom.*;
import javax.swing.JDialog;

/**
 *	Node element
 *
 *	@author Blanc, Yoan
 *	@hearc.copyright 2005 Yoan Blanc, HE-ARC Ingnierie
 *	@hearc.licence <a href="http://opensource.org/licenses/mit-license.php">MIT</a>
 */

public abstract class Node extends Element
{
	/**
	 *	Bound of the Node
	 */
	private Rectangle2D rectangle;
	
	//--
	
	/**
	 *	Build the node
	 *
	 *	@param title		the title of the node
	 */
	public Node(String title)
	{
		super(title);
		
		select(false);
		rectangle = new Rectangle2D.Double(0,0,1,1);
	}
	
	/**
	 *	A copy builder
	 *
	 *	@param n 		the node to copy
	 */
	public Node(Node n)
	{
		super(n);
		
		rectangle = new Rectangle2D.Double();
		rectangle.setRect(n.rectangle);
	}
	
	//--
	
	/**
	 *	Get the top left corner point
	 *
	 *	@return 		the point at the top left corner
	 */
	public Point2D getPoint()
	{
		return new Point2D.Double(rectangle.getX(), rectangle.getY());
	}
	
	/**
	 *	Return the point in the middle of the Node
	 *
	 *	@param g2d	graphic context
	 *	@return		the point in the middle of the Node
	 */
	public Point2D getGravityPoint(Graphics2D g2d)
	{
		calcDimension(g2d);
		return new Point2D.Double(rectangle.getCenterX(), rectangle.getCenterY());
	}
	
	/**
	 *	Get the point touching  the node between the givent point and the middle of the node
	 *
	 *	@param p	distant point
	 *	@param g2d	graphic context
	 *	@return the closest point of the point touching the node
	 */
	public Point2D getProximityPoint(Point2D p, Graphics2D g2d)
	{
		Point2D gravity = getGravityPoint(g2d);
		
		double pl = (p.getY()-gravity.getY()) / (p.getX() - gravity.getX());
		
		double pnode =  (-rectangle.getHeight()) / rectangle.getWidth();
		
		double widthOffset;
		double heightOffset;
		
		if(Math.abs(pl) > -pnode)
		{
			widthOffset = (rectangle.getHeight() / 2.) / pl;
			heightOffset = rectangle.getHeight() / 2.;
		}
		else
		{
			heightOffset = (rectangle.getWidth() / 2.) * pl;
			widthOffset = rectangle.getWidth() / 2.;
		}
		
		if((p.getY() < gravity.getY() && p.getX() < gravity.getX()) ||
			(Math.abs(pl) < -pnode && p.getX() < gravity.getX()) ||
			(Math.abs(pl) > -pnode && p.getY() < gravity.getY()))
		{
			heightOffset = -heightOffset;
			widthOffset = -widthOffset;
		}
		
		return new Point2D.Double(
			gravity.getX() + widthOffset,
			gravity.getY() + heightOffset
		);
	}
	
	/**
	 *	Get the bounds of the Node
	 *
	 *	@return the bounds
	 */
	public Rectangle2D getBounds()
	{
		return rectangle;
	}
	
	/**
	 *	Move the Node
	 *
	 *	@param p			the point
	 */
	public void setPos(Point2D p)
	{
		rectangle.setRect(
			p.getX(),
			p.getY(),
			rectangle.getWidth(),
			rectangle.getHeight()
		);
	}
	
	/**
	 *	Calculate the dimension of the node
	 *
	 *	@param g2d		graphic context
	 */
	public void calcDimension(Graphics2D g2d)
	{
		if(rectangle.getHeight() == 1)
		{
			double w = g2d.getFontMetrics().stringWidth(getTitle()) + getPadding().getWidth() * 2.;
			double h = 20. + getPadding().getHeight() * 2.;
			
			rectangle = new Rectangle2D.Double(rectangle.getX(), rectangle.getY(), w, h);
		}
	}
	
	/**
	 *	Test if the Element contains the point
	 *
	 *	@param p		the point
	 *	@return 		if the Element contains this point
	 */
	public boolean contains(Point2D p)
	{
		return rectangle.contains(p);
	}
	
	/**
	 *	Draw the Node
	 *
	 *	@param g2d	Graphic context
	 */
	public void draw(Graphics2D g2d)
	{		
		calcDimension(g2d);
		
		// background
		g2d.setColor(getBackgroundColor());
		g2d.fill(rectangle);
		
		// foreground
		g2d.setColor(getForegroundColor());
		g2d.draw(rectangle);
		
		// text
		drawTitle(g2d);
	}
	
	/**
	 *	Draw the title's Node
	 *
	 *	@param g2d	Graphic context
	 */
	public void drawTitle(Graphics2D g2d)
	{
		g2d.drawString(
			getTitle(),
			(int)(rectangle.getX() + getPadding().getWidth()),
			(int)(rectangle.getCenterY() + g2d.getFontMetrics().getHeight()/4.)
		);
	}
	
	/**
	 *	Update the values
	 */
	public void update()
	{
		rectangle = new Rectangle2D.Double(rectangle.getX(), rectangle.getY(),0,1);
		super.update();
	}
}
