//
// Copyright (c) 2005 Yoan Blanc, HE-ARC Ingénierie (Le Locle)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
// Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
// OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

package ch.hearc.I04_05s08.GUI;

import javax.swing.*;
import java.awt.*;
import java.awt.geom.*;
import ch.hearc.I04_05s08.GUI.Graph.*;
import ch.hearc.I04_05s08.GUI.RDFS.*;
import ch.hearc.I04_05s08.GUI.RDF.*;

/**
 *	The main application of the system
 *
 *	@author Blanc, Yoan
 *	@hearc.copyright 2005 Yoan Blanc, HE-ARC Ingénierie
 *	@hearc.licence <a href="http://opensource.org/licenses/mit-license.php">MIT</a>
 */

public class MainApplication extends JFrame
{
	/**
	 *	The tabs
	 */
	private JTabbedPane tabs;
	
	/**
	 *	The manager of RDF Schemas
	 */
	private RdfsGraphManager rsgm;
	
	/**
	 *	The manager of RDFs
	 */
	private RdfGraphManager rgm;
	
	/**
	 *	Launch the application
	 */
	public MainApplication()
	{
		// utf-8 !!
		super("Ønt ! (Yoan Blanc - HE-ARC Ingénierie)");
		
		MenuApplication menubar = new MenuApplication();
		MainWindowListener mwl = new MainWindowListener(this);
		
		setJMenuBar(menubar);
		
		rsgm = new RdfsGraphManager(this);
		rgm = new RdfGraphManager(this);
		
		menubar.getRdfsItems()[0].addActionListener(
			new NewFileListener(this, rsgm)
		);
		menubar.getRdfsItems()[1].addActionListener(
			new OpenFileListener(this, rsgm)
		);
		
		menubar.getRdfItems()[0].addActionListener(
			new NewFileListener(this, rgm)
		);
		menubar.getRdfItems()[1].addActionListener(
			new OpenFileListener(this, rgm)
		);
		menubar.getRdfItems()[3].addActionListener(
			new RdfNewFileListener(this, rgm)
		);
		menubar.getRdfItems()[4].addActionListener(
			new RdfOpenFileListener(this, rgm)
		);
		
		menubar.getFileItems()[1].addActionListener(
			new SaveListener(this)
		);
		menubar.getFileItems()[2].addActionListener(
			new SaveAsListener(this)
		);
		menubar.getFileItems()[4].addActionListener(
			new CloseListener(this)
		);
		menubar.getFileItems()[5].addActionListener(
			mwl
		);
		
		menubar.getHelpItem().addActionListener(
			new OpenHelp("User Manual","manual/index.html")
		);
		menubar.getAboutItem().addActionListener(
			new OpenHelp("About","manual/about.html")
		);
		
		menubar.getCopyItem().addActionListener(
			new CopyActionListener(this)
		);
		menubar.getCutItem().addActionListener(
			new CutActionListener(this)
		);
		menubar.getDeleteItem().addActionListener(
			new DeleteActionListener(this)
		);
		
		addWindowListener(
			mwl
		);
		
		tabs = new JTabbedPane();
		getContentPane().add(tabs);
		
		setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
		setSize(640,480);
		centerScreen();
		setVisible(true);
	}
	
	/**
	 *	Add a tab
	 *
	 *	@param tab		a tab
	 */
	public void addTab(Tab tab)
	{
		for(int i=0; i<tabs.getComponentCount(); i++)
		{
			if(((Tab)tabs.getComponentAt(i)).isFile() &&
				((Tab)tabs.getComponentAt(i)).getFile().getFilename().equals(tab.getFile().getFilename()))
			{
				tabs.setSelectedIndex(i);
				return;
			}
		}
		
		tabs.addTab(tab.getTitle(), tab);
		tabs.setSelectedIndex(tabs.getComponentCount()-1);
	}
	
	/**
	 *	Remove the current tab (the active one)
	 */
	public void removeTab()
	{
		tabs.remove(getCurrentTab());
	}
	
	/**
	 *	Get the active tab
	 *
	 *	@return the current tab
	 */
	public Tab getCurrentTab()
	{
		return (Tab)tabs.getSelectedComponent();
	}
	
	/**
	 *	Get the menu bar
	 *
	 *	@return the menu bar
	 */
	public JMenuBar getMenu()
	{
		return getJMenuBar();
	}
	
	/**
	 *	Get the manager of the current tab
	 */
	public GraphManager getManager()
	{
		return (tabs.getComponentCount() > 0
			? getCurrentTab().getManager()
			: null);
	}
	
	/**
	 *	Get the manager of the current tab
	 */
	public GraphPanel getGraph()
	{
		return (tabs.getComponentCount() > 0
			? getCurrentTab().getGraph()
			: null);
	}
	
	/**
	 *	Update the title of the current tab
	 */
	public void updateCurrentTab()
	{
		int pos = tabs.getSelectedIndex();
		tabs.setTitleAt(pos, ((Tab)tabs.getComponentAt(pos)).getTitle());
	}
	
	/**
	 *	Get the rdfs manager
	 *
	 *	@return the manager of rdfs
	 */
	public RdfsGraphManager getRdfsManager()
	{
		return rsgm;
	}
	
	/**
	 *	Get the rdf manager
	 *
	 *	@return the manager of rdf
	 */
	public RdfGraphManager getRdfManager()
	{
		return rgm;
	}
	
	/**
	 *	Center the dialog on the screen
	 */
	public void centerScreen()
	{
		Dimension dim = getToolkit().getScreenSize();
		Rectangle abounds = getBounds();
		setLocation(
			(dim.width - abounds.width) / 2,
			(dim.height - abounds.height) / 2
		);
	}
	
	/**
	 *	Closing all graphs
	 *
	 *	@return if all graph are closed
	 */
	public boolean closeAll()
	{
		boolean val=true;
		
		for(int i=tabs.getComponentCount()-1; i>=0; i--)
		{
			tabs.setSelectedIndex(i);
			if(!((Tab)tabs.getComponentAt(i)).getManager().closeGraph())
			{
				val = false;
				break;
			}
		}
		return val;
	}
}
