//
// Copyright (c) 2005 Yoan Blanc, HE-ARC Ingnierie (Le Locle)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
// Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
// OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

package ch.hearc.I04_05s08.GUI.RDF;

import ch.hearc.I04_05s08.GUI.Graph.*;
import ch.hearc.I04_05s08.Ontology.RDF.*;
import ch.hearc.I04_05s08.Ontology.RDFS.*;
import java.awt.*;

/**
 *	RdfsArrow, an arrow for a RDF graph
 *
 *	@author Blanc, Yoan
 *	@hearc.copyright 2005 Yoan Blanc, HE-ARC Ingnierie
 *	@hearc.licence <a href="http://opensource.org/licenses/mit-license.php">MIT</a>
 */

public class RdfArrow extends Arrow
{
	/**
	 *	The property instanciated
	 */
	private InstanceOfProperty iop;
	
	/**
	 *	The dialog
	 */
	private RdfArrowDialog dialog;
	
	/**
	 *	Init the arrow
	 *
	 *	@param iop	the instance of property
	 *	@param from	the node from
	 *	@param to	the node to
	 */
	public RdfArrow(InstanceOfProperty iop, RdfNode from, RdfNode to)
	{
		super(iop.getShortName(), null, null);
		this.iop = iop;
		setNodes(from, to);
	}
	
	/**
	 *	Init the arrow
	 */
	public RdfArrow(InstanceOfProperty iop)
	{
		this(iop, null, null);
	}
	
	/**
	 *	Build a copy
	 *
	 *	@param a		arrow
	 */
	public RdfArrow(RdfArrow a)
	{
		super(a);
		iop = a.iop;
	}
	
	/**
	 *	Return a clone
	 *	
	 *	@return the cone
	 */
	public Object clone()
	{
		return (Object)new RdfArrow(this);
	}
	
	/**
	 *	Set the nodes
	 *
	 *	@param from		from node
	 *	@param to		to node
	 */
	public boolean setNodes(RdfNode from, RdfNode to)
	{
		boolean val = super.setNodes(from, to);
		
		if(val == true)
		{
			val &= iop.setPropertyOf(from.getInstanceOfClass());
			val &= iop.setProperty(to.getInstanceOfClass());
		}
		
		return val;
	}
	
	/**
	 *	Return the instance
	 *
	 *	@return the instance
	 */
	public InstanceOfProperty getInstanceOfProperty()
	{
		return iop;
	}
	
	/**
	 *	Disconnect the arrow from its nodes
	 *
	 *	@return true (always, cause there is nothing to do here)
	 */
	public boolean unconnect()
	{
		// nothing to do here
		return true;
	}
	
	/**
	 *	Display a dialog box for this node
	 *
	 *	@param graph		the graph
	 */
	public void showDialog(GraphPanel graph)
	{
		super.showDialog(graph);
		
		if(!iop.isInstance() && graph instanceof RdfGraph)
		{
			new RdfArrowDialog(this, (RdfGraph)graph);
		}
	}
	
	/**
	 *	Update the arc
	 */
	public void update()
	{
		setTitle(iop.getShortName());
		if(getGraph() != null)
			((RdfGraph)getGraph()).update();
		super.update();
	}
	
	/**
	 *	Draw the  line and the arrow
	 *
	 *	@param g2d the graphic context
	 */
	public void drawLine(Graphics2D g2d)
	{		
		super.drawLine(g2d);
		
		double r = 20;
		double angle = getAngle();
		
		Polygon arrow = new Polygon(
			new int[]{0, (int)(r* Math.cos(angle + Math.PI/8.)), (int)(r* Math.cos(angle - Math.PI/8.))},
			new int[]{0, (int)(r* Math.sin(angle + Math.PI/8.)), (int)(r* Math.sin(angle - Math.PI/8.))},
			3
		);
		arrow.translate((int)getLine().getP2().getX(), (int)getLine().getP2().getY());
		
		// important for the contains()
		setArrow(arrow);
		
		g2d.setColor(getForegroundColor());
		g2d.fill(arrow);
	}
}
