//
// Copyright (c) 2005 Yoan Blanc, HE-ARC Ingnierie (Le Locle)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
// Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
// OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

package ch.hearc.I04_05s08.GUI.RDFS;

import ch.hearc.I04_05s08.GUI.Graph.*;
import ch.hearc.I04_05s08.Ontology.RDFS.*;
import com.hp.hpl.jena.rdf.model.*;
import java.awt.*;
import java.awt.geom.*;
import javax.swing.JTextPane;
import java.util.Iterator;

/**
 *	The graph for a RDF Schema
 *
 *	@author Blanc, Yoan
 *	@hearc.copyright 2005 Yoan Blanc, HE-ARC Ingnierie
 *	@hearc.licence <a href="http://opensource.org/licenses/mit-license.php">MIT</a>
 */

public class RdfsGraph extends GraphPanel
{
	/**
	 *	RDF Schema
	 */
	private RdfsFile file;
	
	/**
	 *	Source display
	 */
	private JTextPane tp;
	
	/**
	 *	Build
	 *
	 *	@param file			RDF Schema
	 *	@param textpane		text area for displaying sources
	 */
	public RdfsGraph(RdfsFile file, JTextPane textpane)
	{
		super();
		
		if(file != null)
			this.file = file;
		else
			this.file = new RdfsFile("untitled Schema");
	
		tp = textpane;
		update();
	}
	
	/**
	 *	Build
	 *
	 *	@param file		RDF Schema
	 */
	public RdfsGraph(RdfsFile file)
	{
		this(file, null);
	}
	
	/**
	 *	Build
	 *
	 *	@param name 		filename of the RDF Schema
	 */
	public RdfsGraph(String name)
	{
		super();
		
		if(name != null)
			this.file = new RdfsFile(name);
		else
			this.file = new RdfsFile("untitled Schema");
	}
	
	/**
	 *	Add a RDFS node
	 *
	 *	@param n		Node
	 */
	public boolean add(RdfsNode n)
	{
		boolean val = (file.add(n.getResource())
			? super.add((Node)n)
			: false);
		
		update();
		return val;
	}
	
	/**
	 *	Add RDFS nodes
	 *
	 *	@param n		Nodes
	 */
	public boolean add(RdfsNode[] n)
	{
		boolean val = super.add(n);
		
		update();
		return val;
	}
	
	/**
	 *	Add a RDFS arrow
	 *
	 *	@param a		Arrow
	 */
	public boolean add(RdfsArrow a)
	{		
		boolean val = super.add((Arrow)a);
		
		update();
		return val;
	}
	
	/**
	 *	Add RDFS arrows
	 *
	 *	@param a		Arrow
	 */
	public boolean add(RdfsArrow[] a)
	{
		boolean val = super.add(a);
		
		update();
		return val;
	}
	
	/**
	 *	Add an Element to the graph
	 *
	 *	@param e		the element
	 */
	public boolean add(Element e)
	{
		boolean val = true;
		
		if(e instanceof RdfsArrow &&
			getSelectedElement() instanceof RdfsNode &&
			getOldSelectedElement() instanceof RdfsNode)
		{
			if(((RdfsArrow)e).setNodes(
					(RdfsNode)getOldSelectedElement(),
				(RdfsNode)getSelectedElement())
			)
			{
				val = add((RdfsArrow)e.clone());
			}
		}
		else if(e instanceof RdfsNode)
		{
			RdfsNode node = (RdfsNode)e.clone();
			node.showDialog((GraphPanel)this);
			val = add(node);
		}
		
		update();
		return val;
	}
	
	/**
	 *	Remove an arrow
	 *
	 *	@param a		arrow
	 *	@return if the arrow sucessfully removed
	 */
	public boolean del(RdfsArrow a)
	{
		a.unconnect();
		update();
		return super.del(a);
	}
	
	/**
	 *	Delete a node from the schema
	 *
	 *	@param n		the node
	 */
	public boolean del(RdfsNode n)
	{
		boolean val = super.del(n) && file.del(n.getResource());
		update();
		return val;
	}
	
	/**
	 *	Delete nodes from the schema
	 *
	 *	@param n		some nodes
	 */
	public boolean del(RdfsNode[] n)
	{
		boolean val=true;
		
		for(int i=0; i<n.length; i++)
			val &= del(n[i]);
		
		return val;
	}
	
	/**
	 *	Remove some arrows
	 *
	 *	@param a		arrows
	 *	@return if the arrows sucessfully removed
	 */
	public boolean del(RdfsArrow[] a)
	{
		return super.del(a);
	}
	
	/**
	 *	Add an Element to the graph
	 *
	 *	@param e	the element
	 */
	public boolean del(Element e)
	{
		boolean val = true;
		
		if(e instanceof RdfsArrow)
		{
			val = del((RdfsArrow)e);
		}
		else if(e instanceof RdfsNode)
		{
			val = del((RdfsNode)e);
		}
		
		return val;
	}
	
	/**
	 *	Update the graph
	 */
	public void update()
	{
		if(tp != null)
			tp.setText(RdfsManager.rdfsToString(file));
		
		redraw();
	}
	
	/**
	 *	Redraw
	 */
	public void redraw()
	{
		super.redraw();
	}
	
	/**
	 *	Get the node
	 */
	public RdfsNode getNode(RdfsResource res)
	{
		RdfsNode out = null;
		
		for(Iterator i = getNodes().iterator(); i.hasNext();)
		{
			out = (RdfsNode)i.next();
			if(out.getResource().getName().equals(res.getName()))
			{
				return out;
			}
		}
		
		return null;
	}
}
