//
// Copyright (c) 2005 Yoan Blanc, HE-ARC Ingnierie (Le Locle)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
// Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
// OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

package ch.hearc.I04_05s08.Ontology.RDF;

import java.util.LinkedList;
import java.util.Iterator;
import ch.hearc.I04_05s08.Ontology.OntFile;
import ch.hearc.I04_05s08.Ontology.RDFS.RdfsFile;

/**
 *	An instance of some RDF Schema, a RDF File
 *
 *	@see <a href="http://www.w3.org/TR/rdf-schema/#ch_class">w3c</a>
 *	@author Blanc, Yoan
 *	@hearc.copyright 2005 Yoan Blanc, HE-ARC Ingnierie
 *	@hearc.licence <a href="http://opensource.org/licenses/mit-license.php">MIT</a>
 */

public class RdfFile extends OntFile
{
	/**
	 *	Class's instances composing the RDF file
	 */
	private LinkedList classes;
	
	/**
	 *	Property's instances composing the RDF Schema
	 */
	private LinkedList properties;
		
	/**
	 *	RDF Schema's used in the file
	 */
	private RdfsFile[] rdfs;
	
	
	// Constructors ///////////////////////////////////////////////////////////
	
	/**
	 *	Init the RDF file
	 * 
	 *	@param filename		the file's path
	 */
	public RdfFile(String filename)
	{
		this(filename, null);
	}
	
	/**
	 *	Init the untitled RDF file
	 */
	public RdfFile()
	{
		this("", null);
	}
	
	/**
	 *	Init the RDF file
	 * 
	 *	@param filename		the file's path
	 *	@param rdfs			Rdf Schemas
	 */
	public RdfFile(String filename, RdfsFile[] rdfs)
	{
		super(filename);
		
		this.rdfs = rdfs;
		classes = new LinkedList();
		properties = new LinkedList();
	}

	// Public /////////////////////////////////////////////////////////////////
	
	/**
	 *	String representation of the file
	 *
	 *	@return string representation
	 */
	public String toString()
	{
		String str = getFilename();
		
		str += "\n\nClasses :\n";
		for(Iterator c = classes.iterator(); c.hasNext();)
		{
			str += "\n" + ((InstanceOfClass)(c.next())).toString();
		}
		
		str += "\n\nProperties :\n";
		for(Iterator p = properties.iterator(); p.hasNext();)
		{
			str += "\n\n" + ((InstanceOfProperty)(p.next())).toString();
		}
		
		return str;
	}
	
	/**
	 *	Add a new instance of Class to the RDF
	 *
	 *	@param c		the instance class which would be add
	 *	@return if the instance Class wasn't already listed
	 */
	public boolean add(InstanceOfClass c)
	{
		if(c != null && !classes.contains(c))
		{
			return classes.add(c);
		}
		else
		{
			return false;
		}
	}
	
	/**
	 *	Remove a Class from the RDF Schema
	 *
	 *	@param c		the class which would be deleted
	 *	@return if the Class was successfuly removed
	 */
	public boolean del(InstanceOfClass c)
	{
		return classes.remove(c);
	}
	
	/**
	 *	Get all classes members of the RDF Schema
	 *
	 *	@return the class's instances
	 */
	public InstanceOfClass[] getClasses()
	{
		InstanceOfClass[] allclasses;
		Iterator i;
		
		allclasses = new InstanceOfClass[classes.size()];
		
		i = classes.iterator();
		int j=0;
		while(i.hasNext())
		{
			allclasses[j] = (InstanceOfClass)i.next();
			j++;
		}
		
		return allclasses;
	}
	
	/**
	 *	Get a class from his name
	 *
	 *	@param name		class name
	 *	@return a class
	 */
	public InstanceOfClass getClass(String name)
	{
		InstanceOfClass c=null, d;
		for(Iterator i = classes.iterator(); i.hasNext();)
		{
			d= (InstanceOfClass)i.next();
			
			if(d.getValue().equals(name))
			{
				c = d;
				break;
			}
		}
		return c;
	}
	
	/**
	 *	Add a new Property to the RDF Schema
	 *
	 *	@param prop		the property which would be add
	 *	@return if the Property was successfuly added
	 */
	public boolean add(InstanceOfProperty prop)
	{
		if(prop != null && !properties.contains(prop))
		{
			return properties.add(prop);
		}
		else
		{
			return false;
		}
	}
	
	/**
	 *	Delete the instance of Property
	 *
	 *	@param prop		the instance property which would be deleted
	 *	@return if the Property was successfuly deleted
	 */
	public boolean del(InstanceOfProperty prop)
	{
		return properties.remove(prop);
	}
	
	/**
	 *	Get all properties members of the RDF Schema
	 *
	 *	@return the properties
	 */
	public InstanceOfProperty[] getProperties()
	{
		InstanceOfProperty[] allproperties;
		Iterator i;
		
		allproperties = new InstanceOfProperty[properties.size()];
		
		i = properties.iterator();
		int j=0;
		while(i.hasNext())
		{
			allproperties[j] = (InstanceOfProperty)i.next();
			j++;
		}
		
		return allproperties;
	}
	
	/**
	 *	Get a property from his name
	 *
	 *	@param name		property name
	 *	@return a property
	 */
	public InstanceOfProperty getProperty(String name)
	{
		InstanceOfProperty p=null, d;
		for(Iterator i = properties.iterator(); i.hasNext();)
		{
			d= (InstanceOfProperty)i.next();
			if(d.getName().equals(name))
			{
				p = d;
				break;
			}
		}
		return p;
	}
	
	/**
	 *	Get the RDFSchema of the file
	 *
	 *	@return	RDF Schemas
	 */
	public RdfsFile[] getRdfs()
	{
		return rdfs;
	}
}
