//
// Copyright (c) 2005 Yoan Blanc, HE-ARC Ingnierie (Le Locle)
// 
// Permission is hereby granted, free of charge, to any person obtaining a copy of
// this software and associated documentation files (the "Software"), to deal in
// the Software without restriction, including without limitation the rights to use,
// copy, modify, merge, publish, distribute, sublicense, and/or sell copies of the
// Software, and to permit persons to whom the Software is furnished to do so,
// subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all
// copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
// EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
// OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
// IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
//

package ch.hearc.I04_05s08.Ontology.RDF;

import com.hp.hpl.jena.rdf.model.*;
import com.hp.hpl.jena.ontology.*;
import com.hp.hpl.jena.rdql.*;
import com.hp.hpl.jena.vocabulary.RDF;
import java.io.PrintWriter;
import java.io.FileOutputStream;
import java.io.OutputStreamWriter;
import java.io.ByteArrayOutputStream;
import ch.hearc.I04_05s08.Ontology.RDFS.*;

/**
 *	A manager for RDF files
 *
 *	@author Blanc, Yoan
 *	@hearc.copyright 2005 Yoan Blanc, HE-ARC Ingnierie
 *	@hearc.licence <a href="http://opensource.org/licenses/mit-license.php">MIT</a>
 */

public class RdfManager
{
	/**
	 *	Open a RDF file with RDF Schemas
	 *
	 *	@param filename		path of the file
	 *	@param rdfs			RDF Schemas (null for not)
	 *	@return				RDF File
	 */
	public static RdfFile openRdf(String filename, RdfsFile[] rdfs)
	{
		RdfFile file = new RdfFile(filename, rdfs);
		
		Model model = ModelFactory.createDefaultModel();
		model.read(file.getReader(),"");
		file.close();
		
		StmtIterator iter = model.listStatements();
		
		while(iter.hasNext())
		{
			Statement stmt = iter.nextStatement();
			
			InstanceOfProperty p = buildInstanceOfProperty(file, stmt.getPredicate().toString());
			if(p != null)
			{
				InstanceOfClass s = buildInstanceOfClass(file, stmt.getSubject());
				InstanceOfClass o = buildInstanceOfClass(file, stmt.getObject());
				
				p.setPropertyOf(s);
				p.setProperty(o);
			}
			else
			{
				InstanceOfClass s = buildInstanceOfClass(file, stmt.getSubject());
				RdfsClass c = RdfsManager.getClass(rdfs, stmt.getObject().toString());
				s.setClass(c);
			}
			
		}
		
		return file;
	}
	
	/**
	 *	Save a Rdf file
	 *
	 *	@param filename		path of the output
	 *	@param file			rdf file
	 *	@return the state of the operation 
	 */
	public static boolean saveRdf(String filename, RdfFile file)
	{
		Model model = buildModel(file);
		
		try
		{
			model.write(new PrintWriter(
				new OutputStreamWriter(
					new FileOutputStream(filename)
				)
			));
		}
		catch (Exception e)
		{
			System.out.println("error : "+e);
		}
		
		return true;
	}
	
	/**
	 *	Save a rdf file into a string
	 *
	 *	@param file			the rdf file
	 */
	public static String rdfToString(RdfFile file)
	{
		Model model = buildModel(file);
		
		ByteArrayOutputStream baos = new ByteArrayOutputStream();
		
		try
		{
			model.write(new PrintWriter(baos));
		}
		catch (Exception e)
		{
			System.out.println("error : "+e);
		}
		
		return baos.toString();
	}
	
	/**
	 *	Build a model of a rdf file
	 *
	 *	@param file	rdf file
	 *	@return the model of the rdf file
	 */
	public static Model buildModel(RdfFile file)
	{
		// build the Model
		Model model = ModelFactory.createDefaultModel();
		
		// build RDFS
		RdfsFile[] rdfs = file.getRdfs();
		OntModel[] rdfsmodel = null;
		if(rdfs != null)
			 rdfsmodel = new OntModel[rdfs.length];
		
		// Resources
		InstanceOfClass[] classes = file.getClasses();
		Resource[] res = new Resource[classes.length];
		
		// Properties
		InstanceOfProperty[] props = file.getProperties();
		
		try
		{
			for(int i=0; i<props.length; ++i)
			{				
				InstanceOfClass domain = props[i].getPropertyOf();
				InstanceOfClass range = props[i].getProperty();
				
				RDFNode n = null;
				
				if(range != null)
				{
					if(range.isLiteral())
					{
						n = (RDFNode)buildLiteral(range);
					}
					else
					{
						n = (RDFNode)buildResource(range, res, model, rdfs, rdfsmodel);
					}
				}
				
				Property p = buildProperty(props[i]);
				
				if(domain != null && n != null && p != null)
				{
					Resource r = buildResource(domain, res, model, rdfs, rdfsmodel);
					if(r != null)
						r.addProperty(p, n);
				}
			}
			
			for(int i=0; i<classes.length; ++i)
			{
				RDFNode n = null;
				
				if(classes[i].isLiteral())
				{
					n = (RDFNode)buildLiteral(classes[i]);
				}
				else
				{
					n = (RDFNode)buildResource(classes[i], res, model, rdfs, rdfsmodel);
				}
				
				//model.add(n);
			}
		}
		catch(Exception e)
		{
			System.out.println("error (RdfManager.saveRdf): " +e);
		}
		
		return model;
	}
	
	/**
	 *	Build a rdf property
	 *
	 *	@param iop		instance of property
	 *	@return property
	 */
	private static Property buildProperty(InstanceOfProperty iop)
	{
		return ResourceFactory.createProperty(iop.getName());
	}
	
	/**
	 *	Build a rdf literal
	 *
	 *	@param ioc			instance of class
	 *	@return the resource
	 */
	public static Literal buildLiteral(InstanceOfClass ioc)
	{
		return ResourceFactory.createPlainLiteral(ioc.getValue());
	}
	
	/**
	 *	Build a rdf resource
	 *
	 *	@param ioc			instance of class
	 *	@param res			known resource
	 *	@param model			the model's file
	 *	@param rdfs			the RDF Schemas
	 *	@param rdfsmodel	the rdfs model
	 *	@return the resource
	 */
	private static Resource buildResource(InstanceOfClass ioc, Resource[] res, Model model, RdfsFile[] rdfs, OntModel[] rdfsmodel)
	{
		int i=0;		// i is the position of the last empty element in res. (important)
		
		// search
		for(; i<res.length; ++i)
		{
			if(res[i] != null)
			{
				if(!ioc.isLiteral() && res[i].getURI().equals(ioc.getValue()))
				{
					return res[i];
				}
			}
			else
			{
				break;
			}
		}
		
		// not found, build
		res[i] = model.createResource(ioc.getValue());
		if(ioc.getResource() != null)
		{
			res[i].addProperty(RDF.type, model.createResource(ioc.getResource().getName()));
		}
		
		return res[i];
	}
	
	/**
	 *	Build the required OntModel (for a rdfsClass)
	 *
	 *	@param c				the RdfsClass
	 *	@param rdfs			rdfs files
	 *	@param rdfsModel	known Model
	 *	@return the model of the rdfsClass
	 */
	private static OntModel buildOntModel(InstanceOfClass c, RdfsFile[] rdfs, OntModel[] rdfsModel)
	{
		int i=0;
		//search
		if(c.isInstance() && rdfs != null)
		{
			for(;i<rdfs.length;++i)
			{
				if(rdfs[i].containsClass((RdfsClass)c.getResource()))
				{
					if(rdfsModel[i] == null)
					{
						rdfsModel[i] = RdfsManager.buildModel(rdfs[i]);
					}
					break;
				}
			}
		}
		
		return rdfsModel[i];
	}
	
	/**
	 *	Return a class from the rdf file
	 *
	 *	@param file		rdf file
	 *	@param o			object that is instanciate (Literal of RdfNode)
	 */
	private static InstanceOfClass buildInstanceOfClass(RdfFile file, Object o)
	{
		InstanceOfClass ioc;
		
		if (o instanceof Literal)
		{
			ioc = new InstanceOfClass();
			ioc.setValue(((Literal)o).getString());
		}
		else
		{
			String name = o.toString();
			
			ioc = file.getClass(name);
			
			if(ioc == null)
			{
				RdfsClass c = null;
				
				RdfsFile[] rdfs = file.getRdfs();
				if(rdfs != null)
				{
					c = RdfsManager.getClass(rdfs, name);
				}
				
				if(c == null)
				{
					ioc = new InstanceOfClass(name);
				}
				else
				{
					ioc = new InstanceOfClass(c, name);
				}
			}
		}
		
		file.add(ioc);
		
		return ioc;
	}
	
	/**
	 *	Return a property from the rdf file
	 *
	 *	@param file		rdf file
	 *	@param name		identifier of the property (his url)
	 */
	private static InstanceOfProperty buildInstanceOfProperty(RdfFile file, String name)
	{
		InstanceOfProperty iop;
		
		RdfsFile[] rdfs = file.getRdfs();
		if(rdfs != null)
		{
			RdfsProperty p = RdfsManager.getProperty(rdfs, name);
			if(p == null)
			{
				if(name.equals(InstanceOfProperty.type))
				{
					iop = null;
				}
				else
				{
					iop = new InstanceOfProperty(name);
				}
			}
			else
			{
				iop = new InstanceOfProperty(p);
			}
		}
		else
		{
			iop = new InstanceOfProperty(name);
		}
		
		file.add(iop);
		
		return iop;
	}
}
